# Removing records with the following hormonal contraceptives:
# hydroxyprogesterone injection
# micronized progesterone oral
# medroxyprogesterone oral
# estradiol injection
# estradiol-testosterone injection
# conjugated estrogens oral
# estradiol vaginal
# estradiol oral

condHCExclude <- hc$medicationname %in% c('HYDROXPROGESTERONE Injection',
                                          'MICRONIZED PROGESTERONE Oral',
                                          'MEDROXYPROGESTERONE Oral',
                                          'ESTRADIOL Injection',
                                          'ESTRADIOL-TESTOSTERONE Injection',
                                          'CONJUGATED ESTROGENS Oral',
                                          'ESTRADIOL Vaginal',
                                          'ESTRADIOL Oral')
hc <- hc[!condHCExclude,]

#condIUD <- grepl('IUD',hc$form)
#hc <- hc[!condIUD,]

## Added 4/12/2022
# Removing records where hormonal contraception is > 45.0
condstart45 <- !is.na(hc$age_at_med_start) & hc$age_at_med_start > 45.0
hc <- hc[!condstart45,]
###################################


# Data managing hormonal contraceptive medicines:
hc$days_on_med <- 365.25*(hc$age_at_med_stop - hc$age_at_med_start)
hc$depo <- as.numeric(hc$medicationname %in% c('MEDROXYPROGESTERONE Injection','HYDROXYPROGESTERONE Injection'))

# Removing a few records that are clearly in error:
# 1)  994660I where days_on_med < 1
# 2)  999772I where days_on_med < 1
condLT1 <- hc$cfar_pid %in% c('994660I','999772I') & hc$days_on_med < 1
hc <- hc[!condLT1,]

# Two IDs with identical start/stop ages where one needs to be removed:
# 1)  995282K -- drop the patch
# 2)  996993D -- drop ETHINYL ESTRADIOL + LEVONORGESTREL Oral
condDrop1 <- hc$cfar_pid == '995282K' & hc$age_at_med_start == 25.1937 & hc$form == 'Patch'
condDrop2 <- hc$cfar_pid == '996993D' & hc$age_at_med_start == 36.0411 & hc$age_at_med_stop == 40.1561
hc <- hc[!condDrop1 & !condDrop2,]

# Per Jessie, Depo exposure should never be < 3 months (since considering a month as 28 days, then 3 months is 3*28)
condDepoLT3mnths <- hc$depo == 1 & hc$days_on_med < 3*28
hc$depo_exposure_too_short <- 0
hc$depo_exposure_too_short[condDepoLT3mnths] <- 1
hc$age_at_med_stop_updated <- hc$age_at_med_stop
hc$age_at_med_stop_updated[condDepoLT3mnths] <- hc$age_at_med_start[condDepoLT3mnths] + 3*28/365.25

## Added 4/12/2022
# For those HC whose stop date was > 45, changing age_at_med_stop_updated to be 45
condStop45 <- !is.na(hc$age_at_med_stop_updated) & hc$age_at_med_stop_updated > 45
hc$age_at_med_stop_updated[condStop45] <- 45.0
#############################

# Adding in age at first visit and age at last visit:
hc$age_at_first_visit <- patients[match(hc$cfar_pid, patients$cfar_pid, nomatch=NA),'age_at_first_visit']
hc$age_at_lastvisit <- patients[match(hc$cfar_pid, patients$cfar_pid, nomatch=NA),'age_at_lastvisit']

# Exposure to hormonal contraceptives that were completely before the age_at_first_visit need to be exclude:
condBf <- hc$age_at_med_stop < hc$age_at_first_visit
hc <- hc[!condBf,]

# For those who started med before the first visit and stopped it after the first visit, need to update the age at med start to be age at first visit:
condOverlap1stVisit <- hc$age_at_med_start < hc$age_at_first_visit & hc$age_at_med_stop > hc$age_at_first_visit
hc$age_at_med_start_updated <- hc$age_at_med_start
hc$age_at_med_start_updated[condOverlap1stVisit] <- hc$age_at_first_visit[condOverlap1stVisit]

# Modifying a few overlapping HC:
cond1 <- hc$cfar_pid == '997655I' & hc$medicationname == 'NORETHINDRONE Oral' & hc$age_at_med_start_updated == 40.9747
hc$age_at_med_stop_updated[cond1] <- 41.4565

cond2 <- hc$cfar_pid == '999470L' & hc$medicationname == 'MEDROXYPROGESTERONE Injection' & hc$age_at_med_start_updated == 27.7454
hc$age_at_med_stop_updated[cond2] <- 28.0082

cond3 <- hc$cfar_pid == '993058G' & hc$medicationname == 'ETHINYL ESTRADIOL + NORGESTREL Oral' & hc$age_at_med_start_updated == 32.5147
hc$age_at_med_stop_updated[cond3] <- 41.4182

cond4 <- hc$cfar_pid == '993058G' & hc$medicationname == 'ETHINYL ESTRADIOL + NORGESTREL Oral' & hc$age_at_med_start_updated == 33.6126
hc <- hc[!cond4,]

cond5 <- hc$cfar_pid == '981649I' & (hc$age_at_med_start_updated == 32.2546 | hc$age_at_med_start_updated == 32.501)
hc <- hc[!cond5,]

cond6 <- hc$cfar_pid == '981253F' & hc$age_at_med_start_updated == 27.4031
hc$age_at_med_stop_updated[cond6] <- hc$age_at_med_start_updated[cond6] + 5

cond7 <- hc$cfar_pid == '981284A' & hc$age_at_med_start_updated == 33.9055
hc$age_at_med_stop_updated[cond7] <- hc$age_at_med_start_updated[cond7] + 1

cond8 <- hc$cfar_pid == '981706E' & hc$age_at_med_start_updated == 44.7337
hc$age_at_med_stop_updated[cond8] <- hc$age_at_med_start_updated[cond8] + 3

cond9 <- hc$cfar_pid == '982040D' & hc$age_at_med_start_updated == 39.217
hc$age_at_med_stop_updated[cond9] <- hc$age_at_med_start_updated[cond9] + 5

cond10 <- hc$cfar_pid == '996725K' & hc$age_at_med_start_updated == 31.5866
hc$age_at_med_stop_updated[cond10] <- hc$age_at_med_start_updated[cond10] + 3

cond11 <- hc$cfar_pid == '996800I' & hc$age_at_med_start_updated == 28.8077
hc$age_at_med_stop_updated[cond11] <- hc$age_at_med_start_updated[cond11] + 1

cond12 <- hc$cfar_pid == '997648C' & hc$age_at_med_start_updated == 33.6044
hc$age_at_med_stop_updated[cond12] <- hc$age_at_med_start_updated[cond12] + 1

cond13 <- hc$cfar_pid == '998485J' & hc$age_at_med_start_updated == 31.0801
hc$age_at_med_stop_updated[cond13] <- hc$age_at_med_start_updated[cond13] + 3

cond14 <- hc$cfar_pid == '997994A' & hc$age_at_med_start_updated == 37.8453
hc$age_at_med_stop_updated[cond14] <- hc$age_at_med_start_updated[cond14] + 5

cond15 <- hc$cfar_pid == '995134B' & hc$age_at_med_start_updated == 28.6598
hc$age_at_med_start_updated[cond15] <- 28.7392

cond16 <- hc$cfar_pid == '995134B' & hc$age_at_med_start_updated == 29.3908
hc$age_at_med_start_updated[cond16] <- 29.41

cond17 <- hc$cfar_pid == '994772B' & hc$age_at_med_start_updated == 35.3949
hc$age_at_med_start_updated[cond17] <- 35.5072

cond18 <- hc$cfar_pid == '995134B' & hc$age_at_med_start_updated == 29.3908
hc$age_at_med_start_updated[cond18] <- 29.41

cond19 <- hc$cfar_pid == '996351A' & hc$age_at_med_start_updated == 43.9398
hc <- hc[!cond19,]

cond20 <- hc$cfar_pid == '996351A' & hc$age_at_med_start_updated == 44.0739
hc$age_at_med_start_updated[cond20] <- 44.14508

cond21 <- hc$cfar_pid == '996668B' & hc$age_at_med_start_updated == 25.9138
hc <- hc[!cond21,]

cond22 <- hc$cfar_pid == '996993D' & hc$age_at_med_stop_updated == 44.1013
hc$age_at_med_stop_updated[cond22] <- 36.0411

cond23 <- hc$cfar_pid == '998979C' & hc$age_at_med_stop_updated == 23.655
hc$age_at_med_stop_updated[cond23] <- 23.5264

cond24 <- hc$cfar_pid == '981521G' & hc$age_at_med_stop_updated == 18.6092
hc$age_at_med_stop_updated[cond24] <- hc$age_at_med_start_updated[which(cond24==TRUE)+1]

cond25 <- hc$cfar_pid == '993522A' & hc$age_at_med_stop_updated == 23.6934
hc$age_at_med_stop_updated[cond25] <- hc$age_at_med_start_updated[cond25] + 28/365.25

cond26 <- hc$cfar_pid == '993537A' & hc$age_at_med_stop_updated == 40.2053
hc$age_at_med_stop_updated[cond26] <- hc$age_at_med_start_updated[cond26] + 3.0

cond27 <- hc$cfar_pid == '993559G' & hc$age_at_med_stop_updated == 32.2957
hc$age_at_med_stop_updated[cond27] <-  hc$age_at_med_start_updated[cond27] + 25/365.25

cond28 <- hc$cfar_pid == '994115C' & hc$age_at_med_stop_updated == 36.8104
hc$age_at_med_stop_updated[cond28] <- hc$age_at_med_start_updated[cond28] + 28/365.25

cond29 <- hc$cfar_pid == '994360D' & hc$age_at_med_stop_updated == 33.577
hc$age_at_med_stop_updated[cond29] <- hc$age_at_med_start_updated[cond29] + 28/365.25

cond30 <- hc$cfar_pid == '994772B' & hc$age_at_med_stop_updated == 33.0732
hc$age_at_med_stop_updated[cond30] <- hc$age_at_med_start_updated[(which(cond30==TRUE)+1)]

cond31 <- hc$cfar_pid == '994955F' & hc$age_at_med_stop_updated == 23.9343
hc$age_at_med_stop_updated[cond31] <- hc$age_at_med_start_updated[cond31] + 28/365.25

cond32 <- hc$cfar_pid == '995358K' & hc$age_at_med_stop_updated == 18.8775
hc$age_at_med_stop_updated[cond32] <- hc$age_at_med_start_updated[cond32] + 28/365.25

cond33 <- hc$cfar_pid == '995626K' & hc$age_at_med_stop_updated == 27.2936
hc$age_at_med_stop_updated[cond33] <- hc$age_at_med_start_updated[cond33] + 28/365.25

cond34 <- hc$cfar_pid == '995845H' & hc$age_at_med_stop_updated == 28.9719
hc$age_at_med_stop_updated[cond34] <- hc$age_at_med_start_updated[cond34] + 28/365.25

cond35 <- hc$cfar_pid == '996089K' & hc$age_at_med_stop_updated == 28.7721
hc$age_at_med_stop_updated[cond35] <- hc$age_at_med_start_updated[which(cond35 == TRUE)+1]

cond36 <- hc$cfar_pid == '996145J' & hc$age_at_med_stop_updated == 27.9507
hc$age_at_med_stop_updated[cond36] <- hc$age_at_med_start_updated[cond36] + 28/365.25

## Check...
#cond37 <- hc$cfar_pid == '996351A' & hc$age_at_med_stop_updated == 43.9425
#hc$age_at_med_stop_updated[cond37] <- hc$age_at_med_start_updated[which(cond37 == TRUE) + 1]

cond38 <- hc$cfar_pid == '996426C' & hc$age_at_med_stop_updated == 33.7604
hc$age_at_med_stop_updated[cond38] <- hc$age_at_med_start_updated[which(cond38==TRUE)+1]

cond39 <- hc$cfar_pid == '996580H' & hc$age_at_med_stop_updated == 41.2238
hc$age_at_med_stop_updated[cond39] <- hc$age_at_med_start_updated[cond39] + 28/365.25

cond39b <- hc$cfar_pid == '996580H' & hc$age_at_med_stop_updated == 29.3333
hc$age_at_med_stop_updated[cond39b] <- hc$age_at_med_start_updated[cond39b] + 28/365.25

cond40 <- hc$cfar_pid == '996725K' & hc$age_at_med_stop_updated == 21.7988
hc$age_at_med_stop_updated[cond40] <- hc$age_at_med_start_updated[which(cond40==TRUE)+1]

cond41 <- hc$cfar_pid == '996893F' & hc$age_at_med_stop_updated == 36.7118
hc$age_at_med_stop_updated[cond41] <- hc$age_at_med_start_updated[cond41] + 28/365.25

cond42 <- hc$cfar_pid == '997253J' & hc$age_at_med_stop_updated == 37.5661
hc$age_at_med_stop_updated[cond42] <- hc$age_at_med_start_updated[cond42] + 28/365.25

cond43 <- hc$cfar_pid == '998955B' & hc$age_at_med_stop_updated == 23.1923
hc$age_at_med_stop_updated[cond43] <- hc$age_at_med_start_updated[cond43] + 5.0

cond44 <- hc$cfar_pid == '999197I' & hc$age_at_med_stop_updated == 34.1766
hc$age_at_med_stop_updated[cond44] <- hc$age_at_med_start_updated[cond44] + 10.0

cond45 <- hc$cfar_pid == '999275B' & hc$age_at_med_stop_updated == 38.3655
hc$age_at_med_stop_updated[cond45] <- hc$age_at_med_start_updated[cond45] + 28/365.25

cond46 <- hc$cfar_pid == '999772I' & hc$age_at_med_stop_updated == 23.833
hc$age_at_med_stop_updated[cond46] <- hc$age_at_med_start_updated[cond46] + 28/365.25

cond47 <- hc$cfar_pid == '999010J' & hc$age_at_med_stop_updated == 26.141
hc$age_at_med_stop_updated[cond47] <- hc$age_at_med_start_updated[which(cond47 == TRUE)+1]

cond48 <- hc$cfar_pid == '981521G' & hc$age_at_med_stop_updated == 19.1321
hc$age_at_med_stop_updated[cond48] <- 18.6776 # Per email from Megan on 12/17/2020

cond49 <- hc$cfar_pid == '994772B' & hc$medicationname == 'ETHINYL ESTRADIOL + NORGESTIMATE Oral'
hc <- hc[!cond49,]

cond50 <- hc$cfar_pid == '994955F' & hc$medicationname == 'ETHINYL ESTRADIOL + NORGESTREL Oral'
hc <- hc[!cond50,]

cond51 <- hc$cfar_pid == '995626K' & hc$medicationname == 'ETHINYL ESTRADIOL + NORETHINDRONE Oral'
hc <- hc[!cond51,]

cond52 <- hc$cfar_pid == '995845H' & hc$medicationname == 'ETHINYL ESTRADIOL + LEVONORGESTREL Oral'
hc <- hc[!cond52,]

cond53 <- hc$cfar_pid == '996145J' & hc$medicationname == 'ETHINYL ESTRADIOL + NORELGESTROMIN External'
hc <- hc[!cond53,]

cond54 <- hc$cfar_pid == '996725K' & hc$medicationname == 'ETHINYL ESTRADIOL + NORGESTIMATE Oral'
hc <- hc[!cond54,]

cond55 <- hc$cfar_pid == '997253J' & hc$medicationname == 'ETHINYL ESTRADIOL + NORGESTREL Oral'
hc <- hc[!cond55,]

cond56 <- hc$cfar_pid == '999275B' & hc$age_at_med_start_updated == 38.3628
hc$age_at_med_stop_updated[cond56] <- hc$age_at_lastvisit[cond56]
# Making sure all Depo instances are >= 90 days:
# There are 3 IDs impacted by this for depo.1:
# 996123D
# 997651E
# 999732A
condDepo1LT90 <- hc$cfar_pid %in% c('996123D','997651E','999732A') & hc$depo == 1
hc$age_at_med_stop_updated[condDepo1LT90] <- hc$age_at_med_start_updated[condDepo1LT90] + (28*3)/365.25


# Recalculating:
hc$days_on_med <- 365.25*(hc$age_at_med_stop_updated - hc$age_at_med_start_updated)
hc$depo <- as.numeric(hc$medicationname %in% c('MEDROXYPROGESTERONE Injection','HYDROXYPROGESTERONE Injection'))


# Making wide version of data to merge into monthly data:
hc$hc_number <- NA
split(hc$hc_number, hc$cfar_pid) <- lapply(split(hc$cfar_pid, hc$cfar_pid), FUN=function(y){
                                       seq(1,length(y))})

hc$iud <- 0
hc$iud[grep('IUD',hc$form)] <- 1


# Added 4/8/2022:
hc$age_at_lastvisit <- patients[match(hc$cfar_pid, patients$cfar_pid, nomatch=NA),'age_at_lastvisit']
# There are some HC starts that are after the age_at_lastvisit -- need to remove those records
# There are some HC stops that are after the age_at_lastvisit -- need to change the med stop to be age_at_lastvisit
condStartAfterLast <- !is.na(hc$age_at_med_start_updated) & !is.na(hc$age_at_lastvisit) &
                      hc$age_at_med_start_updated >= hc$age_at_lastvisit
hc <- hc[!condStartAfterLast,]

condStopAfterLast <- !is.na(hc$age_at_med_stop_updated) & !is.na(hc$age_at_lastvisit) &
                      hc$age_at_med_stop_updated > hc$age_at_lastvisit
hc$age_at_med_stop_updated[condStopAfterLast] <- hc$age_at_lastvisit[condStopAfterLast]
###############################
wideVars <- Cs(cfar_pid,
               hc_number,
               age_at_med_start_updated,
               age_at_med_stop_updated,
               medicationname,
               days_on_med,
               depo,
               iud)
hc <- as.data.frame(hc)
hc <- hc[order(hc$cfar_pid, hc$age_at_med_start_updated),]

hc_wide <- reshape(hc[,wideVars], v.names = c('age_at_med_start_updated',
                                              'age_at_med_stop_updated',
                                              'medicationname',
                                              'days_on_med',
                                              'depo',
                                              'iud'),
                            idvar = 'cfar_pid',
                            timevar = 'hc_number',
                            direction = 'wide')
hc_wide <- as.data.table(hc_wide)
cond1 <- hc_wide$cfar_pid == '997655I'
hc_wide$age_at_med_start_updated.3[cond1] <- 41.7084
hc_wide$age_at_med_stop_updated.3[cond1] <- 42.3272
hc_wide$medicationname.3[cond1] <- 'NORETHINDRONE Oral'
hc_wide$days_on_med.3[cond1] <- 365.25*(hc_wide$age_at_med_stop_updated.3[cond1] - hc_wide$age_at_med_start_updated.3[cond1])
hc_wide$depo.3[cond1] <- 0

